import os
import pandas as pd
from pydantic import BaseModel
from aryaxai.client.client import APIClient
from aryaxai.common.environment import Environment
from aryaxai.core.organization import Organization
from aryaxai.common.xai_uris import (
    AVAILABLE_BATCH_SERVERS_URI,
    AVAILABLE_CUSTOM_SERVERS_URI,
    AVAILABLE_SYNTHETIC_CUSTOM_SERVERS_URI,
    CLEAR_NOTIFICATIONS_URI,
    CREATE_ORGANIZATION_URI,
    GET_NOTIFICATIONS_URI,
    LOGIN_URI,
    USER_ORGANIZATION_URI,
)
import getpass


class XAI(BaseModel):
    """Base class to connect with AryaXAI platform"""

    env: Environment = Environment()
    api_client: APIClient = APIClient()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        debug = self.env.get_debug()
        base_url = self.env.get_base_url()

        self.api_client = APIClient(debug=debug, base_url=base_url)

    def login(self):
        """login to AryaXAI platform

        :param api_key: API key, defaults to XAI_ACCESS_TOKEN environment variable
        """
        access_token = os.environ.get("XAI_ACCESS_TOKEN", None) or getpass.getpass(
            "Enter your Arya XAI Access Token: "
        )

        if not access_token:
            raise ValueError("Either set XAI_ACCESS_TOKEN or pass the Access token")

        res = self.api_client.post(LOGIN_URI, payload={"access_token": access_token})
        self.api_client.update_headers(res["access_token"])
        self.api_client.set_access_token(access_token)

        print("Authenticated successfully.")

    def organizations(self) -> pd.DataFrame:
        """Get all organizations associated with user

        :return: Organization details dataframe
        """

        res = self.api_client.get(USER_ORGANIZATION_URI)

        if not res["success"]:
            raise Exception(res.get("details", "Failed to get organizations"))

        res["details"].insert(
            0,
            {
                "name": "Personal",
                "organization_owner": True,
                "organization_admin": True,
                "current_users": 1,
                "created_by": "you",
            },
        )

        organization_df = pd.DataFrame(
            res["details"],
            columns=[
                "name",
                "organization_owner",
                "organization_admin",
                "current_users",
                "created_by",
                "created_at",
            ],
        )

        return organization_df

    def organization(self, organization_name: str) -> Organization:
        """Select specific organization

        :param organization_name: Name of the organization to be used
        :return: Organization object
        """
        if organization_name == "personal":
            return Organization(
                api_client=self.api_client,
                **{
                    "name": "Personal",
                    "organization_owner": True,
                    "organization_admin": True,
                    "current_users": 1,
                    "created_by": "you",
                }
            )

        organizations = self.api_client.get(USER_ORGANIZATION_URI)

        if not organizations["success"]:
            raise Exception(organizations.get("details", "Failed to get organizations"))

        user_organization = [
            Organization(api_client=self.api_client, **organization)
            for organization in organizations["details"]
        ]

        organization = next(
            filter(
                lambda organization: organization.name == organization_name,
                user_organization,
            ),
            None,
        )

        if organization is None:
            raise Exception("Organization Not Found")

        return organization

    def create_organization(self, organization_name: str) -> Organization:
        """Create New Organization

        :param organization_name: Name of the new organization
        :return: Organization object
        """
        payload = {"organization_name": organization_name}
        res = self.api_client.post(CREATE_ORGANIZATION_URI, payload)

        if not res["success"]:
            raise Exception(res.get("details", "Failed to create organization"))

        return Organization(api_client=self.api_client, **res["organization_details"])

    def get_notifications(self) -> pd.DataFrame:
        """get user notifications

        :return: notification details dataFrame
        """
        res = self.api_client.get(GET_NOTIFICATIONS_URI)

        if not res["success"]:
            raise Exception("Error while getting user notifications.")

        notifications = res["details"]

        if not notifications:
            return "No notifications found."

        return pd.DataFrame(notifications).reindex(
            columns=["project_name", "message", "time"]
        )

    def clear_notifications(self) -> str:
        """clear user notifications

        :return: response
        """
        res = self.api_client.post(CLEAR_NOTIFICATIONS_URI)

        if not res["success"]:
            raise Exception("Error while clearing user notifications.")

        return res["details"]

    def available_batch_servers(self) -> dict:
        """available custom batch servers

        :return: response
        """
        res = self.api_client.get(AVAILABLE_BATCH_SERVERS_URI)
        return res["details"]

    def available_custom_servers(self) -> dict:
        """available custom servers

        :return: response
        """
        res = self.api_client.get(AVAILABLE_CUSTOM_SERVERS_URI)
        return res

    def available_synthetic_custom_servers(self) -> dict:
        """available synthetic custom servers

        :return: response
        """
        res = self.api_client.get(AVAILABLE_SYNTHETIC_CUSTOM_SERVERS_URI)
        return res["details"]



MODEL_TYPES = ["classification", "regression"]

DATA_DRIFT_DASHBOARD_REQUIRED_FIELDS = [
    "base_line_tag",
    "current_tag",
    "stat_test_name",
]

DATA_DRIFT_STAT_TESTS = [
    "chisquare",
    "jensenshannon",
    "ks",
    "kl_div",
    "psi",
    "wasserstein",
    "z",
]


TARGET_DRIFT_DASHBOARD_REQUIRED_FIELDS = [
    "base_line_tag",
    "current_tag",
    "baseline_true_label",
    "current_true_label",
    "model_type",
    "stat_test_name",
]

TARGET_DRIFT_STAT_TESTS = ["chisquare", "jensenshannon", "kl_div", "psi", "z"]

TARGET_DRIFT_STAT_TESTS_CLASSIFICATION = [
    "chisquare",
    "jensenshannon",
    "kl_div",
    "psi",
]

TARGET_DRIFT_STAT_TESTS_REGRESSION = [
    "jensenshannon",
    "kl_div",
    "ks",
    "psi",
    "wasserstein",
]


BIAS_MONITORING_DASHBOARD_REQUIRED_FIELDS = [
    "base_line_tag",
    "baseline_true_label",
    "baseline_pred_label",
    "model_type",
]

MODEL_PERF_DASHBOARD_REQUIRED_FIELDS = [
    "base_line_tag",
    "current_tag",
    "baseline_true_label",
    "baseline_pred_label",
    "current_true_label",
    "current_pred_label",
    "model_type",
]


DATA_DRIFT_TRIGGER_REQUIRED_FIELDS = [
    "trigger_name",
    "trigger_type",
    "mail_list",
    "frequency",
    "stat_test_name",
    "datadrift_features_per",
    "base_line_tag",
    "current_tag",
]

TARGET_DRIFT_TRIGGER_REQUIRED_FIELDS = [
    "trigger_name",
    "trigger_type",
    "mail_list",
    "frequency",
    "model_type",
    "stat_test_name",
    "baseline_true_label",
    "current_true_label",
    "base_line_tag",
    "current_tag",
]

MODEL_PERF_TRIGGER_REQUIRED_FIELDS = [
    "trigger_name",
    "trigger_type",
    "mail_list",
    "frequency",
    "model_type",
    "model_performance_metric",
    "model_performance_threshold",
    "baseline_true_label",
    "baseline_pred_label",
    "base_line_tag",
]

MODEL_PERF_METRICS_CLASSIFICATION = [
    "accuracy",
    "f1",
    "false_negative_rate",
    "false_positive_rate",
    "precision",
    "recall",
    "true_negative_rate",
    "true_positive_rate",
]

MODEL_PERF_METRICS_REGRESSION = [
    "mean_abs_perc_error",
    "mean_abs_perc_error",
    "mean_squared_error",
    "r2_score",
]

MAIL_FREQUENCIES = [
    "1 hour",
    "3 hour",
    "6 hour",
    "daily",
    "weekly",
    "monthly",
    "quarterly",
    "yearly",
]

SYNTHETIC_MODELS_DEFAULT_HYPER_PARAMS = {
    "GPT2": {
        "batch_size": 250,
        "early_stopping_patience": 10,
        "early_stopping_threshold": 0.11,
        "epochs": 100,
        "model_type": "tabular",
        "random_state": 1,
        "tabular_config": "GPT2Config",
        "train_size": 0.8,
    },
    "CTGAN": {"epochs": 100, "test_ratio": 0.2},
}



import pandas as pd
from pydantic import BaseModel
from typing import Optional
from aryaxai.client.client import APIClient
from aryaxai.common.validation import Validate
from aryaxai.common.xai_uris import (
    AVAILABLE_CUSTOM_SERVERS_URI,
    CREATE_WORKSPACE_URI,
    GET_WORKSPACES_URI,
    INVITE_USER_ORGANIZATION_URI,
    ORGANIZATION_MEMBERS_URI,
    REMOVE_USER_ORGANIZATION_URI,
)
from aryaxai.core.workspace import Workspace


class Organization(BaseModel):
    """Class to work with AryaXAI organizations"""

    organization_id: Optional[str] = None
    name: str
    created_by: str
    created_at: Optional[str] = None

    api_client: APIClient

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.api_client = kwargs.get("api_client")

    def add_user_to_organization(self, user_email: str) -> str:
        """Add user to Organization

        :param user_email: Email of user to be added to organization.
        :return: response
        """
        payload = {
            "email": user_email,
            "organization_id": self.organization_id,
        }
        res = self.api_client.post(INVITE_USER_ORGANIZATION_URI, payload)

        if not res["success"]:
            raise Exception(res.get("details", "Failed to add user to organization"))

        return res.get("details", "User added successfully")

    def remove_user_from_organization(self, user_email: str) -> str:
        """Remove user from Organization

        :param user_email: Email of user to be removed from organization.
        :return: response
        """
        payload = {
            "organization_user_email": user_email,
            "organization_id": self.organization_id,
        }
        res = self.api_client.post(REMOVE_USER_ORGANIZATION_URI, payload)

        if not res["success"]:
            raise Exception(
                res.get("details", "Failed to remove user from organization")
            )

        return res.get("details", "User removed successfully")

    def member_details(self) -> pd.DataFrame:
        """Organization Member details

        :return: member details dataframe
        """
        res = self.api_client.get(
            f"{ORGANIZATION_MEMBERS_URI}?organization_id={self.organization_id}"
        )

        if not res["success"]:
            raise Exception(
                res.get("details", "Failed to get organization member details")
            )

        member_details_df = pd.DataFrame(
            res.get("details").get("users"),
            columns=[
                "full_name",
                "email",
                "organization_owner",
                "organization_admin",
                "created_at",
            ],
        )

        return member_details_df

    def workspaces(self) -> pd.DataFrame:
        """get user workspaces

        :return: workspace details dataframe
        """

        url = GET_WORKSPACES_URI
        if self.organization_id:
            url = url + f"?organization_id={self.organization_id}"
        workspaces = self.api_client.get(url)

        workspace_df = pd.DataFrame(
            workspaces["details"],
            columns=[
                "user_workspace_name",
                "access_type",
                "created_by",
                "created_at",
                "updated_at",
                "instance_type",
                "instance_status",
            ],
        )

        return workspace_df

    def workspace(self, workspace_name: str) -> Workspace:
        """select specific workspace

        :param workspace_name: Name of the workspace to be used
        :return: Workspace
        """

        url = GET_WORKSPACES_URI
        if self.organization_id:
            url = url + f"?organization_id={self.organization_id}"
        workspaces = self.api_client.get(url)
        user_workspaces = [
            Workspace(api_client=self.api_client, **workspace)
            for workspace in workspaces["details"]
        ]

        workspace = next(
            filter(
                lambda workspace: workspace.user_workspace_name == workspace_name,
                user_workspaces,
            ),
            None,
        )

        if workspace is None:
            raise Exception("Workspace Not Found")

        return workspace

    def create_workspace(
        self, workspace_name: str, server_type: Optional[str] = None
    ) -> Workspace:
        """create user workspace

        :param workspace_name: name for the workspace
        :param server_type: dedicated instance to run workloads
            for all available instances check xai.available_custom_servers()
            defaults to shared
        :return: response
        """
        payload = {"workspace_name": workspace_name}

        if server_type:
            custom_servers = self.api_client.get(AVAILABLE_CUSTOM_SERVERS_URI)
            Validate.value_against_list(
                "server_type",
                server_type,
                [server["name"] for server in custom_servers],
            )

            payload["instance_type"] = server_type

        if self.organization_id:
            payload["organization_id"] = self.organization_id

        res = self.api_client.post(CREATE_WORKSPACE_URI, payload)

        if not res["success"]:
            raise Exception(res.get("details"))

        workspace = Workspace(api_client=self.api_client, **res["workspace_details"])

        return workspace

    def __print__(self) -> str:
        return f"Organization(name='{self.name}', created_by='{self.created_by}', created_at='{self.created_at}')"

    def __str__(self) -> str:
        return self.__print__()

    def __repr__(self) -> str:
        return self.__print__()



from typing import List
from pydantic import BaseModel
import pandas as pd

class Alert(BaseModel):
    info: dict
    detailed_report: List[dict]
    not_used_features: List[dict]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # print(self.view_info())
        # print(self.view_detailed_report())
        # print(self.view_not_used_features())
        
    def view_info(self) -> pd.DataFrame:
        """view the alert info

        :return: _description_
        """
        if not self.info:
            return 'There was an error while executing the alert.'

        return pd.DataFrame([self.info])
        
    def view_detailed_report(self) -> pd.DataFrame:
        """view the detailed report of alert

        :return: _description_
        """
        if not self.detailed_report:
            return 'There was an error while executing the alert.'

        detailed_report = [{key: value for key, value in feature.items() if key != 'current_small_hist' and key != 'ref_small_hist'} for feature in self.detailed_report]

        return pd.DataFrame(detailed_report)

    def view_not_used_features(self) -> pd.DataFrame:
        """view the not used features of alert

        :return: _description_
        """
        if not self.not_used_features:
            return 'Not used features is empty.'

        return pd.DataFrame(self.not_used_features)
        
    def __print__(self) -> str:
        return self.view_info()

    def __str__(self) -> str:
        return self.__print__()

    def __repr__(self) -> str:
        return self.__print__()